/***************************************************************************
                     accountmanager.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kconfig.h>
#include <kglobal.h>
#include <kstddirs.h>
#include <kdesktopfile.h>
#include <kdebug.h>

#include "accountmanager.h"
#include "knmprotocol.h"

AccountManager::AccountManager(QObject *parent, const char *name)
  : PluginManager(parent, name),
    _filterManager(this, "filterManager"),
    _messageList(), _deletingAccountList(),
    _checkingAccountList(), _cancelingCheck(false)
  {
  _messageList.setAutoDelete(true);
  
  loadPlugins();
  }

AccountManager::~AccountManager()
  {
  _messageList.clear();
  }  

void AccountManager::checkMessages()
  {
  // If there are no accounts, just call the slotDone, otherwise
  // start all the accounts checking
  if (_pluginList.count() == 0)
    {
      _messageList.clear();
      emit doneChecking(_messageList);
    }

  else
    {
      // Iterator through all accounts and have them start deleting and
      // checking
      QListIterator<KNMPlugin> iter(_pluginList);
      for (iter.toFirst(); iter.current(); ++iter)
	deleteAndCheckMail(*(KNMProtocol*)(iter.current()));
    }

  // Once all the accounts are started, we can clear the message list
  _messageList.clear();
  }

void AccountManager::cancelCheck()
  {
  // Run through all accounts and tell them to cancel
  // We will only tell checking accounts, if the account is deleting,
  // let it finish, so we don't end up with currupted mailboxes
  KNMProtocol *acct;
  QValueList<Q_UINT32>::Iterator iter;
  
  for (iter = _checkingAccountList.begin();
       iter != _checkingAccountList.end(); ++iter)
    {
    acct = (KNMProtocol*)findPlugin(*iter, _pluginList);
    if (acct != NULL)
      acct->cancelCheck();
    }

  _cancelingCheck = true;
  }

void AccountManager::initPlugin(KNMPlugin *plugin)
  {
  KNMProtocol *acct = (KNMProtocol*)plugin;
  connect(acct, SIGNAL(doneChecking(bool, Q_UINT32)),
	  this, SLOT(slotDoneChecking(bool, Q_UINT32)));
  connect(acct, SIGNAL(doneDeleting(bool, Q_UINT32)),
	  this, SLOT(slotDoneDeleting(bool, Q_UINT32)));
  connect(acct, SIGNAL(messageReceived(KNMMessage*)),
	  this, SLOT(slotMessageReceived(KNMMessage*)));
  }

bool AccountManager::isChecking()
  {
  bool checking = false;
  
  // Check if any account is checking or deleting
  checking = ((_deletingAccountList.count() > 0) ||
	      (_checkingAccountList.count() > 0));

  return checking;
  }

void AccountManager::markMessageForDelete(Q_UINT32 msgID)
  {
  // Iterator through the messages and try to find it
  QListIterator<KNMMessage> iter(_messageList);

  for (iter.toFirst(); iter.current(); ++iter)
    {
      if (iter.current()->getUniqueID() == msgID)
	iter.current()->markForDelete();
    }
  }

void AccountManager::unmarkMessageForDelete(Q_UINT32 msgID)
  {
  // Iterator through the messages and try to find it
  QListIterator<KNMMessage> iter(_messageList);

  for (iter.toFirst(); iter.current(); ++iter)
    {
      if (iter.current()->getUniqueID() == msgID)
	iter.current()->unmarkForDelete();
    }
  }

void AccountManager::slotMessageReceived(KNMMessage *msg)
  {
  _messageList.append(msg);
  }

void AccountManager::slotDoneChecking(bool success, Q_UINT32 accountID)
  {
  KNMProtocol *acct = (KNMProtocol*)findPlugin(accountID, _pluginList);
  
  if (!success)
    {
      // there must be an error 
      if (acct != NULL)
	emit errorOccured(acct->getAccountName(),
			  acct->getLastError());
    }

  // remove them from the checking list
  _checkingAccountList.remove(accountID);
  
  if ((_checkingAccountList.count() == 0) &&
      (_deletingAccountList.count() == 0))
    {
    emit doneChecking(_messageList);
    _cancelingCheck = false;
    }
  }

void AccountManager::slotDoneDeleting(bool success, Q_UINT32 accountID)
  {
  KNMProtocol *acct = (KNMProtocol*)findPlugin(accountID, _pluginList);
  
  if (!success)
    {
      // there must be an error
      if (acct != NULL)
	emit errorOccured(acct->getAccountName(),
			  acct->getLastError());
    }

  // remove them from the deleting list
  _deletingAccountList.remove(accountID);

  // if there was no error and we are not canceling,
  // put them on the checking list
  if (success && !_cancelingCheck)
    {
      _checkingAccountList << accountID;
      if (acct != NULL)
	acct->checkMessages();
    }
  else if (_cancelingCheck)
    {
    // If there are no more accounts checking or deleting,
    // emit the signal
    if ((_checkingAccountList.count() == 0) &&
	(_deletingAccountList.count() == 0))
      {
      emit doneChecking(_messageList);
      _cancelingCheck = false;
      }
    }
    
  }

void AccountManager::deleteAndCheckMail(KNMProtocol &acct)
  {
  // get the account uid, then run through all the messages and look
  // for any messages owned by this account and need deleting. Build
  // a list of all these messages, then tell the account to delete them.

  Q_UINT32 uid = acct.getUniqueID();
  KNMMessage *msg = NULL;
  QList<KNMMessage> deadList;
  deadList.setAutoDelete(true);
  
  QListIterator<KNMMessage> iter(_messageList);
  for (iter.toFirst(); iter.current(); /*noop*/)
    {
      msg = iter.current();

      if ((msg->getOwnerUID() == uid) &&
	  (msg->isMarkedForDelete()))
	{
	  _messageList.take(_messageList.findRef(msg));
	  deadList.append(msg);
	}
      else
	++iter;
    }

  if (deadList.count() > 0)
    {
      _deletingAccountList << uid;
      acct.deleteMessages(deadList);
    }
  else
    {
      _checkingAccountList << uid;
      acct.checkMessages();
    }

  // delete all dead messages
  deadList.clear();
  }
