/***************************************************************************
                         pluginmanager.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>

#include <kmainwindow.h>
#include <kglobal.h>
#include <kstddirs.h>
#include <kdesktopfile.h>
#include <kdebug.h>
#include <kconfig.h>

#include "pluginmanager.h"

extern "C" {
  static KNMPlugin* (*pluginInit)(KMainWindow *);
  static KNMPluginInfo* (*pluginInfoInit)(QObject *);
}

PluginManager::PluginManager(QObject *parent, const char *name)
      : QObject(parent, name), _pluginLibDict(), _pluginList()
  {
  _pluginLibDict.setAutoDelete(true);
  _pluginList.setAutoDelete(true);
  }

PluginManager::~PluginManager()
  {
  _pluginList.clear();
  _pluginLibDict.clear();
  }

QStringList PluginManager::getPluginList()
  {
  QStringList plugins;
  
  QDictIterator<KLibrary> iter(_pluginLibDict);
  for (iter.toFirst(); iter.current(); ++iter)
    {
      plugins << iter.currentKey();
    }
  
  return plugins;
  }

KNMPluginInfo *PluginManager::createPluginInfo(const QString &pluginName)
  {
  KNMPluginInfo *info = NULL;
  KLibrary *lib = _pluginLibDict.find(pluginName);
  
  if (lib)
    {
    pluginInfoInit = (KNMPluginInfo* (*)(QObject*))
	(lib->symbol(QString("init") + pluginName + QString("PluginInfo")));

    info = pluginInfoInit(this);
    }

  return info;
  }

KNMPlugin *PluginManager::createPlugin(const QString &pluginName)
  {
  KNMPlugin *plugin = NULL;
  KLibrary *lib = _pluginLibDict.find(pluginName);
  
  if (lib)
    {
      pluginInit = (KNMPlugin* (*)(KMainWindow*))
	(lib->symbol(QString("init") + pluginName + QString("Plugin")));

      if (pluginInit)
	plugin = pluginInit((KMainWindow*)qApp->mainWidget());
    }

  return plugin;
  }

KNMPlugin *PluginManager::findPlugin(Q_UINT32 id, QList<KNMPlugin> &pluginList)
  {
  bool found = false;
  KNMPlugin *target = NULL;
  
  QListIterator<KNMPlugin> iter(pluginList);
  for (iter.toFirst(); iter.current() && !found; ++iter)
    {
      if (iter.current()->getUniqueID() == id)
	{
	  found = true;
	  target = iter.current();
	}
    }

  return target;
  }

void PluginManager::readOptions(KConfig &config)
  {
  _pluginList.clear();

  readPlugins(config, _pluginList);

  QListIterator<KNMPlugin> iter(_pluginList);
  for (iter.toFirst(); iter.current(); ++iter)
    {
      initPlugin(*iter);
    }
  }

void PluginManager::writeOptions(KConfig &config)
  {
  writePlugins(config, _pluginList);
  }

void PluginManager::readPlugins(KConfig &config, QList<KNMPlugin> &pluginList)
  {
  QStringList pluginTypes;
  KNMPlugin *plugin = NULL;
  int counter = 1;
  QString group;
  QString libName;
  
  // Read the list of notifier types
  config.setGroup(getManagedPluginType() + " Manager");
  pluginTypes = config.readListEntry("Types");

  // Now iterate through plugin types, creating and loading the
  // plugins
  QStringList::Iterator iter;
  for (iter = pluginTypes.begin();
       iter != pluginTypes.end();
       ++iter)
    {
    group = getManagedPluginType() + "_" + QString::number(counter);
    config.setGroup(group);

    plugin = createPlugin(*iter); 

    if (plugin)
      {
	plugin->readOptions(config);
	
	pluginList.append(plugin);
      }
    
    else
      {
	emit errorOccured(getManagedPluginType() + "Manager (core)",
			  "Unable to load library for plugin type " + *iter); 
      }
    
    counter++;
    }
  }

void PluginManager::writePlugins(KConfig &config, QList<KNMPlugin> &pluginList)
  {
  QStringList pluginTypes;
  int counter = 1;
  QString group;
  KNMPlugin *plugin = NULL;
  
  QListIterator<KNMPlugin> iter(pluginList);

  for (iter.toFirst(); iter.current(); ++iter)
    {
    plugin = iter.current();
    
    pluginTypes << plugin->getPluginInfo().getPluginType();
    
    group = getManagedPluginType() + "_" + QString::number(counter);
    config.setGroup(group);
    
    plugin->writeOptions(config);

    counter++;
    }

  config.setGroup(getManagedPluginType() + " Manager");
  config.writeEntry("Types", pluginTypes);
  }

void PluginManager::loadPlugins()
  {
  QString name;
  QString libName;
  KLibrary *lib = NULL;
  
  QStringList plugins = KGlobal::dirs()->findAllResources(getManagedResourceType(),
							  "*.desktop");

  kdDebug() << "PluginManager::loadPlugins (" << getManagedPluginType()
	    << "): searching for plugins.\n";
  
  QStringList::Iterator iter;
  for (iter = plugins.begin();
       iter != plugins.end();
       ++iter)
    {
    KDesktopFile df(*iter);

    libName = df.readEntry("X-KDE-Library");
    name = df.readName();

    kdDebug() << "\tloading " << libName << "\n";
    lib = KLibLoader::self()->library(libName);

    if (lib)
      {
      _pluginLibDict.insert(name, lib);
      }
    else
      {
	kdDebug() << "PluginManager::loadPlugins ("<< getManagedPluginType()
		  << "): error loading " << libName << "\n";
      }
    }
  }
