/***************************************************************************
                     knmlocalprotocol.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtextstream.h>

#include <kconfig.h>
#include <kdebug.h>

#include "knmlocalprotocol.h"
#include "localconfigdialog.h"

extern "C" {
  KNMPluginInfo *initLocalPluginInfo(QObject *parent)
    {
    return new KNMPluginInfo("Local", "Local Mail Spool File (Read Only)",
			     "Mike Pilone <mpilone@slac.com>", "1.0.0",
			     parent, "knmlocalprotocolinfo");
    }
  
  KNMProtocol *initLocalPlugin(QObject *parent)
    {
    return new KNMLocalProtocol(parent, "knmlocalprotocol");
    }
}

KNMLocalProtocol::KNMLocalProtocol(QObject *parent, const char *name)
  : KNMProtocol(initLocalPluginInfo(parent), parent, name),
    _spoolFilePath(QString::null), _lastError(QString::null),
    _msgBuffer(QString::null), _spoolFile(), _workTimer(), _numFroms(0)
  {
  connect(&_workTimer, SIGNAL(timeout()), this, SLOT(slotWorkTimeout()));
  }

KNMLocalProtocol::~KNMLocalProtocol()
  {
  }

void KNMLocalProtocol::readOptions(KConfig &config)
  {
  KNMProtocol::readOptions(config);

  // read our own options
  _spoolFilePath = config.readEntry("Spool File", "/var/mail/username");
  }

void KNMLocalProtocol::writeOptions(KConfig &config)
  {
  KNMProtocol::writeOptions(config);

  // write our own options
  config.writeEntry("Spool File", _spoolFilePath);
  }

void KNMLocalProtocol::deleteMessages(QList<KNMMessage> &msgs)
  {
  KNMProtocol::deleteMessages(msgs);
  
  // We don't know hot to delete, so just say we are done
  emit doneDeleting(true, getUniqueID());
  }

void KNMLocalProtocol::checkMessages()
  {
  KNMProtocol::checkMessages();
  
  // We need to be non blocking, so we will open the file here,
  // then start the timer to read from the file. This will ensure
  // the rest of the QT event loop gets processed
  _spoolFile.setName(_spoolFilePath);
  if (!_spoolFile.open(IO_ReadOnly))
    {
    _lastError = "Unable to open ";
    _lastError += _spoolFilePath;
    emit doneChecking(false, getUniqueID());
    }
  else
    {
    // the file is open, start the work timer
    _numFroms = 0;
    _workTimer.start(1, false);
    }
  }

void KNMLocalProtocol::cancelCheck()
  {
  // close the file and stop the timer
  _workTimer.stop();
  if (_spoolFile.isOpen())
    _spoolFile.close();
  }

QString KNMLocalProtocol::getLastError()
  {
  return _lastError;
  }

QString KNMLocalProtocol::getAccountName()
  {
  return _spoolFilePath;
  }

bool KNMLocalProtocol::execConfigDialog(QWidget *parent, const char *name)
  {
  LocalConfigDialog dialog(getPluginName(), _spoolFilePath,
			   parent, name);

  if (dialog.exec())
    {
    _spoolFilePath = dialog.getSpoolFilePath();
    setPluginName(dialog.getPluginName());

    return true;
    }

  return false;
  }

void KNMLocalProtocol::slotWorkTimeout()
  {
  // Read 5 lines of the file
  QString line;
  bool endOfMessage = false;
  
  if (_spoolFile.isOpen())
    {
    QTextStream stream(&_spoolFile);

    for (Q_UINT32 i = 0;
	 (i < (Q_UINT32) 5) && !endOfMessage && !stream.atEnd();
	 i++)
      {
      line = stream.readLine();

      kdDebug() << "Read line: " << line << "\n";
      
      if (line.find("From ", 0, false) == 0)
	{
	_numFroms++;
	if (_numFroms > 1) 
	  endOfMessage = true;
	else
	  {
	  _msgBuffer += line + "\n";
	  }
	}
      else
	{
	_msgBuffer += line + "\n";
	}
      }

    // If we hit the end of a message, or the end of the file and have some
    // data to process, create a message
    if ((endOfMessage) || ((stream.atEnd() && (_msgBuffer.length() > 4))))
      {
      // Process the message
      emit messageReceived(new KNMMessage(getUniqueID(), getAccountName(),
					  _msgBuffer));

      kdDebug() << "processing message: " << _msgBuffer << "\n";
      
      // Add the first line of the new one on there
      _numFroms = 1;
      _msgBuffer = line + "\n";
      }

    // Check if we are at the end of the input
    if (stream.atEnd())
      {
      _workTimer.stop();
      _spoolFile.close();
      emit doneChecking(true, getUniqueID());
      }
    }
  }
